;                  Automatically load the 2000F I/O Processor.
;
;
; This command file loads the 2000F I/O Processor operating system via a
; cross-load from the System Processor.  It is invoked by the SP command files
; to start the IOP and prepare it for data exchange across the Processor
; Interconnect, as follows:
;
;    hp2100 iopload.inc <startup> <cpu-model> <cpu-series>
;
; ...where <startup> is "GEN" or "BOOT" for system generation or system startup,
; respectively, <cpu-model> is the specific CPU model to simulate (2100, 21MX,
; or 21MX-E), and <cpu-series> is the general CPU type (2100 or 21MX).  The
; command file is never invoked directly by the user.
;
;
; NOTES:
;
;  - Concurrent mode is enabled on the simulation console so that the user may
;    examine the IOP configuration without stopping the simulator, which would
;    cause the simulator to exit and, in turn, cause a communications failure
;    at the system processor.


; Capture the I/O Processor startup log.

set -N console log=iopload-%3.log


; Configure the I/O Processor.

do iopconfig.inc %2 %3


; Configure the simulation environment.

set console concurrent


; Configure the multiplexer ports for local or dataset operation.
;
; Ports that are configured for datasets will automatically disconnect their
; associated Telnet sessions if they are not logged on within the timeout set
; by the PHONES command, as well as when the sessions are logged off.  Local
; port Telnet sessions are not disconnected.  As an example, set ports 0 and 1
; for dataset connection.  Additional ports may be specified below as desired.

set MUXL0 DATASET
set MUXL1 DATASET


; Load paper into the line printer if the system is starting.

if "%1" == "BOOT" attach -N LPS printer.txt


; Attach the terminal multiplexer to listen for Telnet connections on localhost
; port 1054.

if "%1" == "BOOT" attach MUX 1054


; Connect the Processor Interconnect cables to the IOP.
;
; The code number here ("4020") is significant only in that the SP instance
; must use the same code number to ensure interprocessor communication.

attach -I IPL 4020


; Load and start the binary loader, which is used as a cross-loader via the IPL
; device.  The loader starts with a CLC 0,C that must execute before the SP
; begins to send data.  With interlocking enabled, this is guaranteed, because
; the first byte read from the photoreader by the SP will take a minimum of 100
; machine instructions, while the CLC 0,C is executed by the IOP in a maximum
; of 10 instructions.  As long as interlocking level is set for 100 or fewer
; instructions, the requirement will be satisfied.
;
; In hardware, an HP 21xx IOP uses the BBL (Basic Binary Loader), whereas an HP
; 21MX IOP uses the paper tape loader ROM (12992K).  In either case, the loader
; is configured to read from the lower processor interconnect select code.
;
; In simulation, we use the IPL device loader, which automatically uses the
; correct loader for the current CPU model (actually, it uses the 2000 Access
; modified BBL for 21xx CPUs).  This permits changing the CPU model above
; without having to redo this section.

echo
echo Receiving the IOP program.

deposit S 000000
boot IPL


; Verify that we had a successful cross-load.

assert T=102077

echo
echo IOP cross-loading completed successfully.


; Initialize the IOP.
;
; The IOP program must complete its initialization before the SP begins to
; communicate.  Otherwise, the system will appear to start up normally, but the
; terminal multiplexer will be unresponsive.
;
; Unfortunately, there is no objective way of determining when initialization
; has completed.  In hardware, the system operator presses RUN on the IOP and
; then presses RUN on the SP.  The inherent human delay in these operations
; ensures that the IOP has executed through its initialization procedure and
; reached the communication loop.
;
; However, with interlocking enabled, this is guaranteed, because the IOP
; completes its initialization (by reaching the start of the communication loop
; at label .COM.) in 263,056 instructions, while the SP issues its first
; communication attempt (sending an Initialize IOP command) after 903,338
; instructions.
;
; The stop-on-I/O-error condition is enabled in the CPU before executing.  When
; the SP exits, its IPL is detached, and this condition is detected as a cable
; break by our IPL device.  Normally, as in hardware, this condition goes
; unreported.  However, by setting the IOERR stop, simulator execution will halt
; with a "Cable not connected" error.  We use this to determine when to
; terminate the IOP instance.

echo
echo The IOP is up and running.
echo

set CPU STOP=IOERR

deposit P 000002
go


; When the SP exits, the IOP simulator stops, and it is safe for us to exit.

exit
